/* Copyright  2009 privacychoice LLC */

const ITrackerWatcher = Components.interfaces.ITrackerWatcher;

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

const eTLDService = Components.classes["@mozilla.org/network/effective-tld-service;1"]
                              .getService(Components.interfaces.nsIEffectiveTLDService);



Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

function TrackerWatcher() {
}

TrackerWatcher.prototype = {
  tabid: null,
  log: function(string) {
    var aConsoleService = Cc["@mozilla.org/consoleservice;1"]
                                 .getService(Ci.nsIConsoleService);   
    aConsoleService.logStringMessage(string);
  },
  
  collecting: false,
  startCollecting: function startCollecting(in_tabid) {
	this.tabid = in_tabid;
	this.collecting = true;
  },
  stopCollecting: function stopCollecting() {
	this.tabid = null;
	this.collecting = false;
	return this.domains;
  },
  getDomains:function getDomains() {
	return this.domains;	
  },
  clearDomains:function getDomains() {
	this.domains = [];
	this.domains_assoc = [];
  },  
  domains: [],
  domains_assoc: [],
  observe: function(subject, topic, data) {
   if (topic == "http-on-modify-request") {
	  if (this.collecting) {
        var httpChannel = subject.QueryInterface(Ci.nsIHttpChannel);
		var host = eTLDService.getBaseDomainFromHost(httpChannel.URI.host);
		 var tabId = this.getTabIDfromDOM(subject, subject);
		if ((this.tabid == tabId) && (!this.domains_assoc[host])) {
          this.domains.push(host);
		  this.domains_assoc[host] = true;
		}
	  }
    }

    if (topic == "app-startup") {
      var os = Components.classes["@mozilla.org/observer-service;1"]
                         .getService(Components.interfaces.nsIObserverService);
      os.addObserver(this, "http-on-modify-request", false);
    }
    return;
  },
  getBrowserWindow: function() {
     return Cc["@mozilla.org/appshell/window-mediator;1"].
            getService(Ci.nsIWindowMediator).
            getMostRecentWindow("navigator:browser");

  },
  getTabIDfromDOM : function(aChannel, aSubject) {
	try {
	  var notificationCallbacks = 
		aChannel.notificationCallbacks ? aChannel.notificationCallbacks : aSubject.loadGroup.notificationCallbacks;
  
	  if (!notificationCallbacks)
		return null;
  
	  var callback = notificationCallbacks.getInterface(Components.interfaces.nsIDOMWindow);
	  if (callback.top.document) {
		var browser = this.getBrowserWindow().gBrowser.getBrowserForDocument(callback.top.document);
		if (browser) {
		  return browser.parentNode.id;
		}
	  }
	}
	catch(e) {}
	return null
  },
  classDescription: "TrackerWatcher Service",
  contractID: "@privacychoice.org/trackerwatcher-service;1",
  classID: Components.ID('{c76e7978-a330-4024-bc7b-bec536e29d47}'),
  QueryInterface: XPCOMUtils.generateQI([Ci.ITrackerWatcher, Ci.nsIObserver]),
  _xpcom_categories: [{
	category: "app-startup",
    service: true
  }]
}

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([TrackerWatcher]);
}
